@@ -1,47 +1,64 @@
 #' Apply running function
 #'
 #' Applies custom function on running windows.
-#' @param x (`vector`, `data.frame`, `matrix`)\cr
+#' @param x (`vector`, `data.frame`, `matrix`, `xts`)\cr
 #'  Input in runner custom function `f`.
 #'
 #' @param k (`integer` vector or single value)\cr
 #'  Denoting size of the running window. If `k` is a single value then window
 #'  size is constant for all elements, otherwise if `length(k) == length(x)`
 #'  different window size for each element. One can also specify `k` in the same
-#'  way as by in \code{\link[base]{seq.POSIXt}}. More in details.
+#'  way as `by` argument in \code{\link[base]{seq.POSIXt}}.
+#'  See 'Specifying time-intervals' in details section.
 #'
 #' @param lag (`integer` vector or single value)\cr
 #'  Denoting window lag. If `lag` is a single value then window lag is constant
 #'  for all elements, otherwise if `length(lag) == length(x)` different window
 #'  size for each element. Negative value shifts window forward. One can also
-#'  specify `lag` in the same way as by in \code{\link[base]{seq.POSIXt}}.
-#'  More in details.
+#'  specify `lag` in the same way as `by` argument in
+#'  \code{\link[base]{seq.POSIXt}}. See 'Specifying time-intervals' in details
+#'  section.
 #'
 #' @param idx (`integer`, `Date`, `POSIXt`)\cr
 #'  Optional integer vector containing sorted (ascending) index of observation.
 #'  By default `idx` is index incremented by one. User can provide index with
-#'  varying increment and with duplicated values. If specified then `k` and `lag`
-#'  are depending on `idx`. Length of `idx` have to be equal of length `x`.
+#'  varying increment and with duplicated values. If specified then `k` and
+#'  `lag` are depending on `idx`. Length of `idx` have to be equal of length
+#'  `x`.
 #'
 #' @param f (`function`)\cr
-#' Applied on windows created from `x`. This function is meant to summarize
-#' windows and create single element for each window, but one can also specify
-#' function which return multiple elements (runner output will be a list).
-#' By default runner returns windows as is (`f = function(x)`).
+#'  Applied on windows created from `x`. This function is meant to summarize
+#'  windows and create single element for each window, but one can also specify
+#'  function which return multiple elements (runner output will be a list).
+#'  By default runner returns windows as is (`f = function(x)`).
 #'
 #' @param at (`integer`, `Date`, `POSIXt`, `character` vector)\cr
 #'  Vector of any size and any value defining output data points. Values of the
 #'  vector defines the indexes which data is computed at. Can be also `POSIXt`
-#'  sequence increment \code{\link[base]{seq.POSIXt}}. More in details.
+#'  sequence increment used in `at` argument in \code{\link[base]{seq.POSIXt}}.
+#'  See 'Specifying time-intervals' in details section.
 #'
 #' @param na_pad (`logical` single value)\cr
 #'  Whether incomplete window should return `NA` (if `na_pad = TRUE`)
 #'  Incomplete window is when some parts of the window are out of range.
 #'
 #' @param type (`character` single value)\cr
-#'  output type (`"auto"`, `"logical"`, `"numeric"`, `"integer"`, `"character"`).
-#'  `runner` by default guess type automatically. In case of failure of `"auto"`
-#'  please specify desired type.
+#'  output type
+#'  (`"auto"`, `"logical"`, `"numeric"`, `"integer"`, `"character"`). `runner`
+#'  by default guess type automatically. In case of failure of `"auto"` please
+#'  specify desired type.
+#'
+#' @param simplify (`logical` or `character` value)\cr
+#'  should the result be simplified to a vector, matrix or higher dimensional
+#'  array if possible. The default value, `simplify = TRUE`, returns a vector or
+#'  matrix if appropriate, whereas if `simplify = "array"` the result may be an
+#'  array of “rank” `(=length(dim(.)))` one higher than the result of output
+#'  from the function `f` for each window.
+#'
+#' @param cl (`cluster`) *experimental*\cr
+#'  Create and pass the cluster to the `runner` function to run each window
+#'  calculation in parallel. See \code{\link[parallel]{makeCluster}} in details.
+#'
 #'
 #' @param ... (optional)\cr
 #'   other arguments passed to the function `f`.
@@ -68,43 +85,101 @@
 #'    If one specifies `idx` this would mean that output windows size might
 #'    change in size because of unequally spaced indexes. Fox example 5-period
 #'    window is different than 5-element window, because 5-period window might
-#'    contain any number of observation (7-day mean is not the same as 7-element mean)
+#'    contain any number of observation (7-day mean is not the same as 7-element
+#'    mean)
 #'     \cr
 #'    \if{html}{\figure{runningdatewindows.png}{options: width="75\%" alt="Figure: runningdatewindows.png"}}
 #'    \if{latex}{\figure{runningdatewindows.pdf}{options: width=7cm}}
 #'  }
 #'  \item{**Window at specific indices**}{\cr
-#'    `runner` by default returns vector of the same size as `x` unless one specifies
-#'    `at` argument. Each element of `at` is an index on which runner calculates function -
-#'    which means that output of the runner is now of length equal to `at`. Note
-#'    that one can change index of `x` by specifying `idx`.
-#'    Illustration below shows output of `runner` for `at = c(18, 27, 45, 31)`
-#'    which gives windows in ranges enclosed in square brackets. Range for `at = 27` is
-#'    `[22, 26]` which is not available in current indices. \cr
+#'    `runner` by default returns vector of the same size as `x` unless one
+#'    specifies `at` argument. Each element of `at` is an index on which runner
+#'    calculates function - which means that output of the runner is now of
+#'    length equal to `at`. Note that one can change index of `x` by specifying
+#'    `idx`. Illustration below shows output of `runner` for
+#'    `at = c(18, 27, 45, 31)` which gives windows in ranges enclosed in square
+#'    brackets. Range for `at = 27` is `[22, 26]` which is not available in
+#'    current indices. \cr
 #'    \if{html}{\figure{runnerat.png}{options: width="75\%" alt="Figure: runnerat.png"}}
 #'    \if{latex}{\figure{runnerat.pdf}{options: width=7cm}}
-#'    \cr
-#'    `at` can also be specified as interval of the output defined by `at = "<increment>"`
-#'    which results in output on following indices
-#'    `seq.POSIXt(min(idx), max(idx), by = "<increment>")`. Increment of sequence is the
-#'    same as in \code{\link[base]{seq.POSIXt}} function.
-#'    It's worth noting that increment interval can't be more frequent than
-#'    interval of `idx` - for `Date` the most frequent time-unit is a `"day"`,
-#'    for `POSIXt` a `sec`.
-#'
-#'    `k` and `lag` can also be specified as using time sequence increment. Available
-#'    time units are `"sec", "min", "hour", "day", "DSTday", "week", "month", "quarter" or "year"`.
-#'    To increment by number of units one can also specify `<number> <unit>s`
-#'    for example `lag = "-2 days"`, `k = "5 weeks"`.
 #'  }
 #' }
-#' Above is not enough since `k` and `lag` can be a vector which allows to
-#' stretch and lag/lead each window freely on in time (on indices).
+#' ## Specifying time-intervals
+#'  `at` can also be specified as interval of the output defined by
+#'  `at = "<increment>"` which results in indices sequence defined by
+#'  `seq.POSIXt(min(idx), max(idx), by = "<increment>")`. Increment of sequence
+#'  is the same as in \code{\link[base]{seq.POSIXt}} function.
+#'  It's worth noting that increment interval can't be more frequent than
+#'  interval of `idx` - for `Date` the most frequent time-unit is a `"day"`,
+#'  for `POSIXt` a `sec`.
 #'
-#' @return vector with aggregated values for each window. Length of output is the
-#'  same as `length(x)` or `length(at)` if specified. Type of the output
+#'  `k` and `lag` can also be specified as using time sequence increment.
+#'  Available time units are
+#'`"sec", "min", "hour", "day", "DSTday", "week", "month", "quarter" or "year"`.
+#'  To increment by number of units one can also specify `<number> <unit>s`
+#'  for example `lag = "-2 days"`, `k = "5 weeks"`.
+#'
+#'  Setting `k` and `lag` as a sequence increment can be also a vector can be a
+#'  vector which allows to stretch and lag/lead each window freely on in time
+#'  (on indices).
+#' \cr
+#' ## Parallel computing
+#'  Beware that executing R call in parallel not always
+#'  have the edge over single-thread even if the
+#'  `cl <- registerCluster(detectCores())` was specified before.
+#'  \cr
+#'  Parallel windows are executed in the independent environment, which means
+#'  that objects other than function arguments needs to be copied to the
+#'  parallel environment using \code{\link[parallel]{clusterExport}}`. For
+#'  example using `f = function(x) x + y + z` will result in error as
+#'  \code{clusterExport(cl, varlist = c("y", "z"))} needs to be called before.
+#'
+#' @return vector with aggregated values for each window. Length of output is
+#'  the same as `length(x)` or `length(at)` if specified. Type of the output
 #'  is taken from `type` argument.
 #'
+#' @md
+#' @rdname runner
+#' @importFrom methods is
+#' @importFrom parallel clusterExport parLapply
+#' @export
+runner <- function(
+  x,
+  f = function(x) x,
+  k = integer(0),
+  lag = integer(1),
+  idx = integer(0),
+  at = integer(0),
+  na_pad = FALSE,
+  type = "auto",
+  simplify = TRUE,
+  cl = NULL,
+  ...
+  ) {
+  if (!is.null(cl) && type != "auto") {
+    warning(
+      "There is no option to specify the type of the output using type in parallel mode. #nolint
+      Please use 'simplify' instead"
+    )
+    type <- "auto"
+  }
+  if (!isFALSE(simplify) && type != "auto") {
+    warning(
+      "When 'simplify != FALSE' 'type' argument is set to 'auto'"
+    )
+    type <- "auto"
+  }
+  if (type != "auto") {
+    warning(
+      "Argument 'type'is deprecated and will be defunct in the next release.
+    Please use 'simplify' argument to manage the output type."
+    )
+  }
+
+  UseMethod("runner", x)
+}
+
+#' @rdname runner
 #' @examples
 #'
 #' # runner returns windows as is by default
@@ -169,11 +244,8 @@
 #'   at = c(18, 27, 48, 31),
 #'   f = mean
 #' )
-#' @md
-#' @rdname runner
-#' @importFrom methods is
 #' @export
-runner <- function (
+runner.default <- function(  #nolint
   x,
   f = function(x) x,
   k = integer(0),
@@ -182,14 +254,115 @@ runner <- function (
   at = integer(0),
   na_pad = FALSE,
   type = "auto",
+  simplify = TRUE,
+  cl = NULL,
   ...
-  ) {
-  UseMethod("runner", x)
+) {
+  if (any(is.na(k))) {
+    stop("Function doesn't accept NA values in k vector");
+  }
+  if (any(is.na(lag))) {
+    stop("Function doesn't accept NA values in lag vector");
+  }
+  if (any(is.na(idx))) {
+    stop("Function doesn't accept NA values in idx vector");
+  }
+  if (!is(f, "function")) {
+    stop("f should be a function")
+  }
+
+  # use POSIXt.seq
+  at <- seq_at(at, idx)
+  k <- k_by(k, if (length(at > 0)) at else idx, "k")
+  lag <- k_by(lag, if (length(at > 0)) at else idx, "lag")
+
+  w <- window_run(
+    x = x,
+    k = k,
+    lag = lag,
+    idx = idx,
+    at = at,
+    na_pad = na_pad
+  )
+
+  if (!is.null(cl) && is(cl, "cluster")) {
+    answer <- parLapply(
+      cl = cl,
+      X = w,
+      fun = f,
+      ...
+    )
+
+  } else if (type != "auto") {
+    n <- length(w)
+    answer <- vector(mode = type, length = n)
+    for (i in seq_len(n)) {
+      ww <- w[[i]]
+      answer[i] <- if (length(ww) == 0) {
+        NA
+      } else {
+        f(ww, ...)
+      }
+    }
+
+  } else {
+    answer <- lapply(w, function(.this_window)
+      if (is.null(.this_window)) {
+        NA
+      } else {
+        f(.this_window, ...)
+      }
+    )
+  }
+
+  if (!isFALSE(simplify) && length(answer) && type == "auto") {
+    simplify2array(answer, higher = (simplify == "array"))
+  } else {
+    answer
+  }
 }
 
 #' @rdname runner
+#' @examples
+#'
+#' # runner with data.frame
+#' df <- data.frame(
+#'   a = 1:13,
+#'   b = 1:13 + rnorm(13, sd = 5),
+#'   idx = seq(Sys.Date(), Sys.Date() + 365, by = "1 month")
+#' )
+#' runner(
+#'   x = df,
+#'   idx = "idx",
+#'   at = "6 months",
+#'   f = function(x) {
+#'     cor(x$a, x$b)
+#'   }
+#' )
+#'
+#' # parallel computing
+#' library(parallel)
+#' data <- data.frame(
+#'   a = runif(100),
+#'   b = runif(100),
+#'   idx = cumsum(sample(rpois(100, 5)))
+#' )
+#' const <- 0
+#' cl <- makeCluster(1)
+#' clusterExport(cl, "const", envir = environment())
+#'
+#' runner(
+#'   x = data,
+#'   k = 10,
+#'   f = function(x) {
+#'     cor(x$a, x$b) + const
+#'   },
+#'   idx = "idx",
+#'   cl = cl
+#' )
+#' stopCluster(cl)
 #' @export
-runner.data.frame <- function(
+runner.data.frame <- function( #nolint
   x,
   f = function(x) x,
   k = integer(0),
@@ -198,13 +371,11 @@ runner.data.frame <- function(
   at = integer(0),
   na_pad = FALSE,
   type = "auto",
+  simplify = TRUE,
+  cl = NULL,
   ...
-  ) {
-
-  # dplyr::group_by exception
-  x <- this_group(x)
-
-  # set arguments from attrs set by run_by
+) {
+  # set arguments from attrs (set by run_by)
   k <- set_from_attribute_difftime(x, k) # no deep copy
   lag <- set_from_attribute_difftime(x, lag)
   idx <- set_from_attribute_index(x, idx)
@@ -224,6 +395,120 @@ runner.data.frame <- function(
     stop("f should be a function")
   }
 
+  # use POSIXt.seq
+  at  <- seq_at(at, idx)
+  k   <- k_by(k, if (length(at) > 0) at else idx, "k")
+  lag <- k_by(lag, if (length(at) > 0) at else idx, "lag")
+
+  w <- window_run(
+    x = seq_len(nrow(x)),
+    k = k,
+    lag = lag,
+    idx = idx,
+    at = at,
+    na_pad = na_pad
+  )
+
+  answer <- if (!is.null(cl) && is(cl, "cluster")) {
+    clusterExport(cl, varlist = c("x", "f"), envir = environment())
+    parLapply(
+      cl = cl,
+      X = w,
+      fun = function(.this_window_idx) {
+        if (length(.this_window_idx) == 0) {
+          NA
+        } else {
+          f(x[.this_window_idx, ], ...)
+        }
+      }
+    )
+
+  } else {
+    lapply(w, function(.this_window_idx) {
+      if (length(.this_window_idx) == 0) {
+        NA
+      } else {
+        f(x[.this_window_idx, ], ...)
+      }
+    })
+  }
+
+  if (!isFALSE(simplify) && length(answer)) {
+    simplify2array(answer, higher = (simplify == "array"))
+  } else {
+    answer
+  }
+}
+
+#' @rdname runner
+#' @export
+runner.grouped_df <- function(
+  x,
+  f = function(x) x,
+  k = integer(0),
+  lag = integer(1),
+  idx = integer(0),
+  at = integer(0),
+  na_pad = FALSE,
+  type = "auto",
+  simplify = TRUE,
+  cl = NULL,
+  ...
+) {
+  runner.data.frame(
+    x = this_group(x),
+    f = f,
+    lag = lag,
+    idx = idx,
+    at = at,
+    na_pad = na_pad,
+    type = type,
+    simplify = simplify,
+    cl = cl,
+    ...
+  )
+}
+
+#' @rdname runner
+#' @examples
+#'
+#' # runner with matrix
+#' data <- matrix(data = runif(100, 0, 1), nrow = 20, ncol = 5)
+#' runner(
+#'   x = data,
+#'   f = function(x) {
+#'     tryCatch(
+#'       cor(x),
+#'       error = function(e) NA
+#'     )
+#'  })
+#' @export
+runner.matrix <- function(
+  x,
+  f = function(x) x,
+  k = integer(0),
+  lag = integer(1),
+  idx = integer(0),
+  at = integer(0),
+  na_pad = FALSE,
+  type = "auto",
+  simplify = TRUE,
+  cl = NULL,
+  ...
+) {
+  if (any(is.na(k))) {
+    stop("Function doesn't accept NA values in k vector");
+  }
+  if (any(is.na(lag))) {
+    stop("Function doesn't accept NA values in lag vector");
+  }
+  if (any(is.na(idx))) {
+    stop("Function doesn't accept NA values in idx vector");
+  }
+  if (!is(f, "function")) {
+    stop("f should be a function")
+  }
+
   # use POSIXt.seq
   at  <- seq_at(at, idx)
   k   <- k_by(k,   if (length(at) > 0) at else idx, "k")
@@ -238,20 +523,41 @@ runner.data.frame <- function(
     na_pad = na_pad
   )
 
-  res <- sapply(w, function(ww) {
-    if (length(ww) == 0) {
-      NA
-    } else {
-      f(x[ww, ], ...)
-    }
-  })
-
-  return(res)
+  answer <- if (!is.null(cl) && is(cl, "cluster"))  {
+    clusterExport(cl, varlist = c("x", "f"), envir = environment())
+    parLapply(
+      cl = cl,
+      X = w,
+      fun = function(.this_window_idx) {
+        if (length(.this_window_idx) == 0) {
+          NA
+        } else {
+          f(x[.this_window_idx, , drop = FALSE], ...)
+        }
+      },
+      ...
+    )
+  } else {
+    lapply(
+      X = w,
+      FUN = function(.this_window_idx) {
+      if (length(.this_window_idx) == 0) {
+        NA
+      } else {
+        f(x[.this_window_idx, , drop = FALSE], ...)
+      }
+    })
+  }
+  if (!isFALSE(simplify) && length(answer)) {
+    simplify2array(answer, higher = (simplify == "array"))
+  } else {
+    answer
+  }
 }
 
 #' @rdname runner
 #' @export
-runner.default <- function(
+runner.xts <- function(
   x,
   f = function(x) x,
   k = integer(0),
@@ -260,75 +566,65 @@ runner.default <- function(
   at = integer(0),
   na_pad = FALSE,
   type = "auto",
+  simplify = TRUE,
+  cl = NULL,
   ...
-  ) {
-
-  if (any(is.na(k))) {
-    stop("Function doesn't accept NA values in k vector");
-  }
-  if (any(is.na(lag))) {
-    stop("Function doesn't accept NA values in lag vector");
-  }
-  if (any(is.na(idx))) {
-    stop("Function doesn't accept NA values in idx vector");
-  }
-  if (!is(f, "function")) {
-    stop("f should be a function")
+) {
+  if (!identical(idx, integer(0))) {
+    warning(
+      "'idx' argument has been specified and will mask index
+      of the 'xts' object."
+    )
+  } else {
+    idx <- structure(
+      .Data = as.vector(attr(x, "index")),
+      class = attr(attr(x, "index"), "tclass"),
+      tz = attr(attr(x, "index"), "tzone")
+    )
   }
 
-
-  # use POSIXt.seq
-  at <- seq_at(at, idx)
-  k <- k_by(k, if (length(at > 0)) at else idx, "k")
-  lag <- k_by(lag, if (length(at > 0)) at else idx, "lag")
-
-  w <- window_run(
+  runner.matrix(
     x = x,
+    f = f,
     k = k,
     lag = lag,
     idx = idx,
     at = at,
-    na_pad = na_pad
+    na_pad = na_pad,
+    type = type,
+    simplify = simplify,
+    cl,
+    ...
   )
-
-  if (type != "auto") {
-    n <- length(w)
-    res <- vector(mode = type, length = n)
-    for (i in seq_len(n)) {
-      ww <- w[[i]]
-      res[i] <- if (length(ww) == 0) {
-        NA
-      } else {
-        f(ww, ...)
-      }
-    }
-
-  } else {
-    res <- sapply(w, function(ww)
-      if (is.null(ww)) {
-        NA
-      } else {
-        f(ww, ...)
-      }
-    )
-  }
-
-  return(res)
 }
 
-get_parent_call_arg_names <- function() {
-  cl <- sys.call(-2)
-  f <- get(as.character(cl[[1]]), mode="function", sys.frame(-2))
-  cl <- match.call(definition=f, call=cl)
+# utilities -----
+get_runner_call_arg_names <- function() {
+  runner_call_idx <- which(
+    vapply(
+      X =  rev(sys.calls()),
+      FUN = function(x) x[[1]] == as.name("runner"),
+      FUN.VALUE = logical(1)
+    )
+  ) - 1
+
+  cl <- sys.call(-runner_call_idx)
+  f <- get(
+    x = as.character(cl[[1]]),
+    mode = "function",
+    envir = sys.frame(-runner_call_idx)
+  )
+  cl <- match.call(definition = f, call = cl)
   names(cl)
 }
 
 is_datetime_valid <- function(x) {
   all(
-    grepl("^(sec|min|hour|day|DSTday|week|month|quarter|year)$", x = x) |
-      grepl("^-*[0-9]+ (sec|min|hour|day|DSTday|week|month|quarter|year)s", x = x)
+    grepl("^(sec|min|hour|day|DSTday|week|month|quarter|year)$",
+          x = x) |
+      grepl("^-*[0-9]+ (sec|min|hour|day|DSTday|week|month|quarter|year)s",
+            x = x)
   )
-
 }
 
 
@@ -339,7 +635,11 @@ is_datetime_valid <- function(x) {
 #' @param param name of the parameter to be printed in error message
 #' @examples
 #' k <-  "1 month"
-#' idx <- seq(as.POSIXct("2019-01-01 03:02:01"), as.POSIXct("2020-01-01 03:02:01"), by = "month")
+#' idx <- seq(
+#'   as.POSIXct("2019-01-01 03:02:01"),
+#'   as.POSIXct("2020-01-01 03:02:01"),
+#'   by = "month"
+#' )
 #' k_difftime <- runner:::k_by(k, idx, param = "k")
 #' idx - k_difftime
 k_by <- function(k, idx, param) {
@@ -353,7 +653,10 @@ k_by <- function(k, idx, param) {
     from <- if (length(k) != 1) {
       if (length(idx) == 0) {
         stop(
-          sprintf("`idx` can't be empty while specifying `%s` as time interval", param)
+          sprintf(
+            "`idx` can't be empty while specifying `%s` as time interval",
+            param
+          )
         )
       }
 
@@ -367,7 +670,10 @@ k_by <- function(k, idx, param) {
     } else if (length(k) == 1) {
       if (length(idx) == 0) {
         stop(
-          sprintf("`idx` can't be empty while specifying `%s` as time interval", param)
+          sprintf(
+            "`idx` can't be empty while specifying `%s` as time interval",
+            param
+          )
         )
       }
 
@@ -406,9 +712,13 @@ k_by <- function(k, idx, param) {
 
 #' Formats time-unit-interval to valid for runner
 #'
-#' Formats time-unit-interval to valid for runner
+#' Formats time-unit-interval to valid for runner. User specifies \code{k} as
+#' positive number but this means that this interval needs to be substracted
+#' from \code{idx} - because windows length extends window backwards in time.
+#' The same situation for lag.
 #' @param k (k or lag) object from runner to be formatted
-#' @param only_positive for \code{k} is \code{TRUE}, for \code{lag} is \code{FALSE}
+#' @param only_positive for \code{k} is \code{TRUE},
+#'   for \code{lag} is \code{FALSE}
 #' @examples
 #' runner:::reformat_k("1 days")
 #' runner:::reformat_k("day")
@@ -430,64 +740,13 @@ reformat_k <- function(k, only_positive = TRUE) {
   return(k)
 }
 
-#' Set window parameters
-#'
-#' Set window parameters for \link{runner}. This function sets the
-#' attributes to \code{x} (only \code{data.frame}) object and saves user effort
-#' to specify window parameters in further multiple \link{runner} calls.
-#' @inheritParams runner
-#' @return x object which \link{runner} can be executed on.
-#' @examples
-#' library(dplyr)
-#'
-#' data <- data.frame(
-#'  index = c(2, 3, 3, 4, 5, 8, 10, 10, 13, 15),
-#'  a = rep(c("a", "b"), each = 5),
-#'  b = 1:10
-#' )
-#'
-#' data %>%
-#'  group_by(a) %>%
-#'  run_by(idx = "index", k = 5) %>%
-#'  mutate(
-#'    c = runner(
-#'      x = .,
-#'      f = function(x) {
-#'        paste(x$b, collapse = ">")
-#'      }
-#'    ),
-#'    d = runner(
-#'      x = .,
-#'      f = function(x) {
-#'        sum(x$b)
-#'      }
-#'    )
-#'  )
-#' @export
-run_by <- function(x, idx, k, lag, na_pad, at) {
-  if (!is.data.frame(x)) {
-    stop("`run_by` should be used only for `data.frame`. \n
-         Use `runner` on x directly.")
-  }
-
-  if (!missing(k)) x <- set_run_by_difftime(x, k)
-  if (!missing(lag)) x <- set_run_by_difftime(x, lag)
-  if (!missing(idx)) x <- set_run_by_index(x, idx)
-  if (!missing(at)) x <- set_run_by_index(x, at)
-  if (!missing(na_pad)) attr(x, "na_pad") <- na_pad
-
-  return(x)
-}
-
-
-
 
 #' Creates sequence for at as time-unit-interval
 #'
 #' Creates sequence for at as time-unit-interval
 #' @param at object from runner
 #' @param idx object from runner
-seq_at <- function(at, idx) {
+seq_at <- function(at, idx) { # nolint
   if (length(at) == 1 &&
       (
         (is.character(at) && is_datetime_valid(at)) ||
@@ -514,123 +773,8 @@ seq_at <- function(at, idx) {
   return(at)
 }
 
-set_run_by_index <- function(x, arg) {
-  arg_name <- deparse(substitute(arg))
-
-  attr(x, arg_name) <- if (is.character(arg) && length(arg) == 1 && arg %in% names(x)) {
-    arg
-
-  } else if (is.numeric(arg) || inherits(arg, c("Date", "POSIXct", "POSIXxt", "POSIXlt"))) {
-    arg
-  } else {
-    stop(
-      sprintf(
-        "`%s` should be either:
-         - column name of `x`
-         - vector of type `numeric`, `Date`, `POSIXct` or `POSIXlt`",
-        arg_name
-      ),
-      call. = FALSE
-    )
-  }
-  return(x)
-}
-
-set_run_by_difftime <- function(x, arg) {
-  arg_name <- deparse(substitute(arg))
-
-  attr(x, arg_name) <- if (is.character(arg)) {
-    if (length(arg) == 1 && arg %in% names(x)) {
-      arg
-    } else if (all(is_datetime_valid(arg))) {
-      arg
-    } else {
-      stop(
-        sprintf(
-          "`%s` is invalid, should be either:
-           - column name of `x`
-           - `difftime` class or character describing diffitme (see at argument in `seq.POSIXt`)
-           - `numeric` or `integer` vector",
-          arg_name
-        ),
-        call. = FALSE
-      )
-    }
-  } else if (is.numeric(arg) || is(arg, "difftime")) {
-    arg
-  } else {
-    stop(
-      sprintf(
-        "`%s` is invalid, should be either:
-           - column name of `x`
-           - `difftime` class or character describing diffitme (see at argument in `seq.POSIXt`)
-           - `numeric` or `integer` `vector`",
-        arg_name
-      ),
-      call. = FALSE
-    )
-  }
-  return(x)
-}
-
-set_from_attribute_index <- function(x, attrib) {
-  runner_args <- get_parent_call_arg_names()
-  arg_name <- deparse(substitute(attrib))
-
-  # no arg overwriting
-  if (!is.null(attr(x, arg_name)) && !arg_name %in% runner_args) {
-    if (length(attr(x, arg_name)) == 1 &&
-        is.character(attr(x, arg_name)) &&
-        attr(x, arg_name) %in% names(x)) {
-
-      attrib <- x[[attr(x, arg_name)]]
-    } else if (is.character(attr(x, arg_name))) {
-      stop(
-        sprintf(
-          "`%s` should be either:
-         - column name of `x`
-         - vector of type `numeric`, `Date`, `POSIXct` or `POSIXlt`",
-          arg_name
-        ),
-        call. = FALSE
-      )
-    } else {
-      attrib <- attr(x, arg_name)
-    }
-
-    # arg overwriting (runner masks run_by)
-  } else {
-    if (!is.null(attr(x, arg_name))) {
-      warning(
-        sprintf(
-          "`%1$s` set in run_by() will be ignored in favour of `%1$s` specified in runner() call",
-          arg_name
-        )
-      )
-    }
-
-    if (is.character(attrib) && length(attrib) == 1 && attrib %in% names(x)) {
-      attrib <- x[[attrib]]
-    } else if (is.numeric(attrib) || inherits(attrib, c("Date", "POSIXct", "POSIXxt", "POSIXlt"))) {
-      # do nothing
-    } else {
-      stop(
-        sprintf(
-          "`%s` should be either:
-         - column name of `x`
-         - vector of type `numeric`, `Date`, `POSIXct` or `POSIXlt`",
-          arg_name
-        ),
-        call. = FALSE
-      )
-    }
-  }
-
-  return(attrib)
-}
-
-set_from_attribute_at <- function(x, attrib) {
-  runner_args <- get_parent_call_arg_names()
+set_from_attribute_at <- function(x, attrib) { #nolint
+  runner_args <- get_runner_call_arg_names()
   arg_name <- deparse(substitute(attrib))
 
   # no arg overwriting
@@ -646,7 +790,7 @@ set_from_attribute_at <- function(x, attrib) {
           "`%s` should be either:
          - column name of `x`
          - vector of type `numeric`, `Date`, `POSIXct` or `POSIXlt`
-         - character value describing dates sequence step as in `by` argument of `seq.POSIXct`",
+         - character value describing dates sequence step as in `by` argument of `seq.POSIXct`", #nolint
           arg_name
         ),
         call. = FALSE
@@ -660,7 +804,7 @@ set_from_attribute_at <- function(x, attrib) {
     if (!is.null(attr(x, arg_name))) {
       warning(
         sprintf(
-          "`%1$s` set in run_by() will be ignored in favour of `%1$s` specified in runner() call",
+          "`%1$s` set in run_by() will be ignored in favour of `%1$s` specified in runner() call", #nolint
           arg_name
         )
       )
@@ -670,7 +814,8 @@ set_from_attribute_at <- function(x, attrib) {
       attrib <- x[[attrib]]
     } else if (length(attrib) == 1 && all(is_datetime_valid(attrib))) {
       # do nothing
-    } else if (is.numeric(attrib) || inherits(attrib, c("Date", "POSIXct", "POSIXxt", "POSIXlt"))) {
+    } else if (is.numeric(attrib) ||
+               inherits(attrib, c("Date", "POSIXct", "POSIXxt", "POSIXlt"))) {
       # do nothing
     } else {
       stop(
@@ -689,24 +834,26 @@ set_from_attribute_at <- function(x, attrib) {
 }
 
 
-set_from_attribute_difftime <- function(x, attrib) {
-  runner_args <- get_parent_call_arg_names()
+set_from_attribute_difftime <- function(x, attrib) { #nolint
+  runner_args <- get_runner_call_arg_names()
   arg_name <- deparse(substitute(attrib))
 
-  # no arg overwriting
   if (!is.null(attr(x, arg_name)) && !arg_name %in% runner_args) {
+    #  - argument has not been specified so it can be overwritten
     if (length(attr(x, arg_name)) == 1 && attr(x, arg_name) %in% names(x)) {
+      # attr is a variable name
       attrib <- x[[attr(x, arg_name)]]
     } else {
+      # attr is a vector of values - length validation later
       attrib <- attr(x, arg_name)
     }
 
-  # arg overwriting (runner masks run_by)
-  } else {
+  } else if (arg_name %in% runner_args) {
+    # - argument has been specified
     if (!is.null(attr(x, arg_name))) {
       warning(
         sprintf(
-          "`%1$s` set in run_by() will be ignored in favour of `%1$s` specified in runner() call",
+          "`%1$s` set in run_by() will be ignored in favour of `%1$s` specified in runner() call", #nolint
           arg_name
         )
       )
@@ -714,15 +861,16 @@ set_from_attribute_difftime <- function(x, attrib) {
 
     if (is.character(attrib)) {
       if (length(attrib) == 1 && attrib %in% names(x)) {
+        # argument as variable name
         attrib <- x[[attrib]]
       } else if (all(is_datetime_valid(attrib))) {
-        # do nothing
+        # argument as a difftime character
       } else {
         stop(
           sprintf(
             "`%s` is invalid, should be either:
            - column name of `x`
-           - difftime class or character describing diffitme (see at argument in seq.POSIXt)
+           - difftime class or character describing diffitme (see at argument in seq.POSIXt) #nolint
            - numeric or integer vector",
             arg_name
           ),
@@ -736,7 +884,7 @@ set_from_attribute_difftime <- function(x, attrib) {
         sprintf(
           "`%s` is invalid, should be either:
            - column name of `x`
-           - difftime class or character describing diffitme (see at argument in `seq.POSIXt`)
+           - difftime class or character describing diffitme (see at argument in `seq.POSIXt`) #nolint
            - numeric or integer vector",
           arg_name
         ),
@@ -748,8 +896,66 @@ set_from_attribute_difftime <- function(x, attrib) {
   return(attrib)
 }
 
+set_from_attribute_index <- function(x, attrib) { #nolint
+  arg_name <- deparse(substitute(attrib))
+  runner_args <- get_runner_call_arg_names()
+
+  # No arg overwriting
+  #  - attribute not empty and argument not specified
+  if (!is.null(attr(x, arg_name)) && !arg_name %in% runner_args) {
+    if (length(attr(x, arg_name)) == 1 &&
+        is.character(attr(x, arg_name)) &&
+        attr(x, arg_name) %in% names(x)) {
+
+      attrib <- x[[attr(x, arg_name)]]
+    } else if (is.character(attr(x, arg_name))) {
+      stop(
+        sprintf(
+          "`%s` should be either:
+         - column name of `x`
+         - vector of type `numeric`, `Date`, `POSIXct` or `POSIXlt`",
+          arg_name
+        ),
+        call. = FALSE
+      )
+    } else {
+      attrib <- attr(x, arg_name)
+    }
+
+  # arg overwriting (runner masks run_by)
+  } else {
+    if (!is.null(attr(x, arg_name))) {
+      warning(
+        sprintf(
+          "`%1$s` set in run_by() will be ignored in favour of `%1$s` specified in runner() call", # nolint
+          arg_name
+        )
+      )
+    }
+
+    if (is.character(attrib) && length(attrib) == 1 && attrib %in% names(x)) {
+      attrib <- x[[attrib]]
+    } else if (is.numeric(attrib) ||
+               inherits(attrib, c("Date", "POSIXct", "POSIXxt", "POSIXlt"))) {
+      # do nothing
+    } else {
+      stop(
+        sprintf(
+          "`%s` should be either:
+         - column name of `x`
+         - vector of type `numeric`, `Date`, `POSIXct` or `POSIXlt`",
+          arg_name
+        ),
+        call. = FALSE
+      )
+    }
+  }
+
+  return(attrib)
+}
+
 set_from_attribute_logical <- function(x, attrib) {
-  runner_args <- get_parent_call_arg_names()
+  runner_args <- get_runner_call_arg_names()
   arg_name <- deparse(substitute(attrib))
 
   # no arg overwriting
@@ -760,7 +966,7 @@ set_from_attribute_logical <- function(x, attrib) {
   } else if (!is.null(attr(x, arg_name))) {
     warning(
       sprintf(
-        "`%1$s` set in run_by() will be ignored in favour of `%1$s` specified in runner() call",
+        "`%1$s` set in run_by() will be ignored in favour of `%1$s` specified in runner() call",  #nolint
         arg_name
       )
     )
@@ -772,33 +978,31 @@ set_from_attribute_logical <- function(x, attrib) {
 
 #' Access group data in mutate
 #'
-#' Access group data in `dplyr::mutate` after `dplyr::group_by`.
-#' Function created because data available in `dplyr::group_by %>% mutate` scheme
-#' is not filtered by group - in mutate function `.` is still initial dataset.
-#' This function creates `data.frame` using `dplyr::groups` information.
+#' Access group data in `dplyr::mutate` after
+#' `dplyr::group_by`.
+#' Function created because data available in `dplyr::group_by %>% mutate`
+#' scheme is not filtered by group - in mutate function `.` is still initial
+#' dataset. This function creates `data.frame` using `dplyr::groups`
+#' information.
 #' @param x (`data.frame`)\cr
 #'   object which can be `grouped_df` in special case.
 #' @md
 #' @return data.frame filtered by current `dplyr::groups()`
 this_group <- function(x) {
-  if (is(x, "grouped_df")) {
-    attrs <- attributes(x)
-    attrs <- attrs[names(attrs) != "row.names"]
+  attrs <- attributes(x)
+  attrs <- attrs[names(attrs) != "row.names"]
 
-    new_env <- new.env(parent = parent.frame(n = 2)$.top_env)
-    df_call <- as.call(
-      append(
-        as.name("data.frame"),
-        lapply(names(x), as.name)
-      )
+  new_env <- new.env(parent = parent.frame(n = 2)$.top_env)
+  df_call <- as.call(
+    append(
+      as.name("data.frame"),
+      lapply(names(x), as.name)
     )
+  )
 
-    x <- eval(df_call, envir = new_env)
-    for (i in seq_along(attrs)) {
-      attr(x, names(attrs)[i]) <- attrs[[i]]
-    }
+  x <- eval(df_call, envir = new_env)
+  for (i in seq_along(attrs)) {
+    attr(x, names(attrs)[i]) <- attrs[[i]]
   }
-
   return(x)
 }
-
